/***************************************************************************** 
*             CUD C/C++ Developement Kit v2.0 for Celerity v2.0              *
*                          written by Sledge Hammer                          *
*      These functions have been tested with TC++ 3.0 and BC++ v3.1          *
*    All functions in this file (c) 1993 C.U.D. INC. All rights reserved     *
*****************************************************************************/

#include <dir.h>
#include <dos.h>
#include <fcntl.h>
#include <io.h>
#include <stdio.h>

/* The following are macros for the new PRN function */

#define C      0
#define PASCAL 1

void ctop(char *in, char *out);
void cur_dir(char *path);
void length(char *filename, long *size);
void no_trail_bs(char *any_str);
void PackTime (long *l, struct ftime *o); 
void parse_clock(double t, int *min, int *sec);
void prn(char *string, int type);
void ptoc(char *in,char *out);
void putblock(int ul_x, int ul_y, int lr_x, int lr_y, char *LABEL);
void rem_pipe(char *in, char *out);
void timer(long t1, long t2, int *min, int *sec);
void UnPackTime(long *l, struct ftime *o);

/*****************************************************************************
* Function : ctop v2.0                                                       *
* Description : This function converts 'in' (C type string) to 'out' (Pascal *
*               type string)                                                 *
*                                                                            *
* Format :                                                                   *
*                                                                            *
*   sprintf(cstring,"This is a C type string");                              *
*   ctop(cstring,PascalString);                                              *
*****************************************************************************/

void ctop(char *in, char *out)
 {
  sprintf(out,"%c%s",strlen(in),in);
 }

/*****************************************************************************
* Function : ptoc v2.0                                                       *
* Description : This function converts 'in' (Pascal type string) to 'out' (C *
*               type string)                                                 *
*                                                                            *
* Format :                                                                   *
*                                                                            *
*   ptoc(cstring,PascalString);                                              *
*****************************************************************************/

void ptoc(char *in, char *out)
 {
  int loop;
  for (loop = 1; loop <= in[0]; loop++)
     {
      out[loop-1] = in[loop];
     }  
  out[in[0]] = '\0';
 }

/*****************************************************************************
* Function : UnPackTime                                                      *
* Description : This function will unpack a long variable in the timepack    *
*               format used by Pascal to a format that can be handled by C   *
*                                                                            *
* Format :                                                                   *
*                                                                            *
*   struct ftime t;                                                          *
*   UnPackTime(&record.When,&t);                                             *
*   printf("%d",t.ft_year);                                                  *
*****************************************************************************/

void UnPackTime(long *l, struct ftime *o)
 {
  o->ft_tsec = *l & 31;
  o->ft_min = (*l >> 5) & 63;
  o->ft_hour = (*l >> 11) & 31;
  o->ft_day = (*l >> 16) & 31;
  o->ft_month = (*l >> 21) & 15;
  o->ft_year = (*l >> 25) & 255;
 }

/*****************************************************************************
* Function : PackTime                                                        *
* Description : This function will pack the current system time/date struct  *
*               into the timepack format used by Pascal                      *
*                                                                            *
* Format :                                                                   *
*                                                                            *
*   struct ftime t;                                                          *
*   PackTime(&t,&record.When);                                               *
*   printf("%ld",record.When);                                               *
*****************************************************************************/

void PackTime (long *l, struct ftime *o)
 {
  memcpy(l,o,4);
 }

/*****************************************************************************
* Function : get_packdate                                                    *
* Description : This function will fill the ftime struct used by C with the  *
*               current system date/time, to be used with the PackTime       *
*               function                                                     *
*                                                                            *
* Format :                                                                   *
*                                                                            *
*   struct ftime t;                                                          *
*   get_packdate(&t);                                                        *
*****************************************************************************/

int get_packdate(struct ftime *ft)
 {
  FILE *stream;

  if ((stream = fopen("TIME.$$$","wt")) == NULL)
    return 1;

  getftime(fileno(stream), ft);
  fclose(stream);
  remove("TIME.$$$");
  return 0;
 }

/*****************************************************************************
* Function : rem_pipe                                                        *
* Description : This function will take 'in' of c type and remove the pipe   *
*               dots and copy it to 'out'.                                   *
*                                                                            *
* Format :                                                                   *
*                                                                            *
*   rem_pipe(PStringIn,PStringOut);                                          *
*****************************************************************************/

void rem_pipe(char *in, char *out)
 {
  int loop ,counter = 0;
  for (loop = 0; loop <= strlen(in); loop++)
     {
      if (in[loop] == '|') loop++;
	 else 
	   {
	    out[counter] = out[loop]; counter++; 
	   }
     }
 }

/*****************************************************************************
* Function : prn v2.0                                                        *
* Description : This function will output strings interpreting pipe codes    *
*               along the way.  Version 2.0 has combined the prn and pla     *
*               functions into one routine.                                  *
*                                                                            *
* Format :                                                                   *
*                                                                            *
* For 'C' type strings :                                                     *
*                                                                            *
*   prn(CstringIn,C);                                                        *
*                                                                            *
* For 'Pascal' type strings :                                                *
*                                                                            *
*   prn(PascalStringIn,PASCAL);                                              *
*                                                                            *
*****************************************************************************/

void prn(char *string, int type)
 {
  int loop = 0, length;
  
  if (type == C) length = strlen(string); 
  if (type == PASCAL) 
    {
     loop++;
     length = string[0] + 1;
    }
  while (loop <= length - 1)
       {
	if (string[loop] == '|')
	  {
	   switch (string[loop+1])
		 {
		  case 'B': textcolor(9);
			    break;
		  case 'b': textcolor(1);
			    break;
		  case 'C': textcolor(11);
			    break;
		  case 'c': textcolor(3);
			    break;
		  case 'G': textcolor(10);
			    break;
		  case 'g': textcolor(2);
			    break;
		  case 'M': textcolor(13);
			    break;
		  case 'm': textcolor(5);
			    break;
		  case 'R': textcolor(12);
			    break;
		  case 'r': textcolor(4);
			    break;
		  case 'W': textcolor(15);
			    break;
		  case 'w': textcolor(7);
			    break;
		  case 'Y': textcolor(14);
			    break;
		  case 'y': textcolor(6);
			    break;
		  case '0': textcolor(0);
			    break;
		 }
	   loop++;
	  }
	else cprintf("%c",string[loop]); loop++;
       }
 }

/*****************************************************************************
* Function : block                                                           *
* Description : This function will take the rectangle specified by the       *
*               upper left hand corner (x1,y1) and the lower left hand       *
*               corner (x2,y2) and change the attributes of the text in      *
*               that rectangle.                                              *
*                                                                            *
* Format :                                                                   *
*                                                                            *
*   int newattr;                                                             *
*   newattr(BLACK + (GREEN<<4));                                             *
*   block (1,1,10,10,newattr);                                               *
*****************************************************************************/

void block (int x1, int y1, int x2, int y2, int attr)
 {
  int i = 0,j = 0;
  unsigned char far *v;
  unsigned char far *vid_mem = MK_FP(0xb800,0x0000);
  for (i = y1 - 1; i < y2; i++)
     for (j = x1 - 1; j < x2; j++)
	{ 
	 v = vid_mem;
	 v += (i*160) + j*2;
	 v++;
	 *v = attr;
	}
 }

/*****************************************************************************
* Function : putblock                                                        *
* Description : A nice feature of TheDraw (tm) is that it will allow you     *
*               to design screens and save them in C format to allow you to  *
*               implement them in your program, however, the authors of      *
*               TheDraw (tm) never supplied us with a function to put an     *
*               ACTUAL block on the screen with out overwriting outside of   *
*               that block. That is what this function is for. It is a       *
*               "modified" version of puttext, that will allow you to do     *
*               exactly this.  It is called in the same way as puttext.      *
*                                                                            *
* Format :                                                                   *
*                                                                            *
*   #include "image.h"                                                       *
*   putblock(14,8,70,19,IMAGE);                                              *
*****************************************************************************/

void putblock(int ul_x, int ul_y, int lr_x, int lr_y, char *LABEL)
 {
  void far *screen; 
  int offset = (((ul_x * 2) + (ul_y * 160)) - 162), x = 0; 
  
  for (x = 0; x <= (lr_y - ul_y); x++)  
     {
      (long) screen = 0xB8000000+x*160+offset; 
      memcpy (screen,&LABEL[x*80*2],(lr_x-ul_x)*2); 
     }
 }

/*****************************************************************************
* Function : chk_file                                                        *
* Description : This checks if a certain file exists.  If a file exists it   *
*               returns 1 otherwise a 0 is returned.                         *
*                                                                            *
* Format :                                                                   *
*                                                                            *
*   chkfile("TEST.$$$");                                                     *
*****************************************************************************/

int chk_file(char *filename)
 { 
  return (access(filename,0) == 0);
 }

/*****************************************************************************
* Function : chk_dir                                                         *
*                                                                            *
* Description : This checks if a path exists.  Returns -1 if the path doesnt *
*               exist, 0 if the path does exist.                             *
* Format :                                                                   *
*                                                                            *
* chk_dir("C:\DOS");                                                         *
*****************************************************************************/

int chk_dir(char *check_this)
 {
  int a, original_drive, disk;
  char path[200], curdir[MAXPATH];
  cur_dir(curdir);
  strcpy(path,check_this);
  no_trail_bs(path);
  if ((path[1] == 58) && (path[2] == 0))
    {
     original_drive = getdisk();
     disk = (path[0] - 'A');
     setdisk(disk);
     if (disk == getdisk()) a = 0;
     else a = -1;
     setdisk(original_drive);
    }
  else
   {
    a = chdir(path);
   }
 if (a != 0)
   {
    return -1;
   }
 else
   {
    chdir(curdir); return 0;
   };
 }

void no_trail_bs(char *any_str)
 {
  int  count;
  
  for (count = strlen(any_str) - 1; count >= 0; count--)
     {
      if (any_str[count] == 32 || any_str[count] == 92) any_str[count] = 0;
      else break;
     }
 }

void cur_dir(char *path)
 {
  strcpy(path,"X:\\");
  path[0] = 'A' + getdisk();
  getcurdir(0, path+3);
 }

/*****************************************************************************
* Function : length                                                          *
*                                                                            *
* Description : This returns the length of a file.                           *
*                                                                            *
* Format :                                                                   *
*                                                                            *
* long size;                                                                 *
* length("C:\DOS\COMMAND.COM",&size);                                        *
*****************************************************************************/

void length(char *filename, long *size)
 {
  int handle = open(filename,O_RDONLY | O_BINARY);
  *size = filelength(handle);
  close(handle);
 }

/*****************************************************************************
* Function : parse clock                                                     *
*                                                                            *
* Description : This parses out the double value returned by difftime into a *
*               minute and second format                                     *
*                                                                            *
* Format :                                                                   *
*                                                                            *
* parse_clock(diff,&min, &sec);                                              *
*****************************************************************************/

void parse_clock(double t, int *min, int *sec)
 {
  if (t == 60) 
    {
     *min = 1; 
     *sec = 0;
    }
  if (t > 60)
    {
     *min = t / 60;
     *sec = t - (*min*60);
    }
  if (t < 60) 
    {
     *min = 0;
     *sec = t;
    }
 }

/*****************************************************************************
* Function : Create_file                                                     *
*                                                                            *
* Description : This creates a new file                                      *
*                                                                            *
* Format :                                                                   *
*                                                                            *
* creat_file("C:\TEMP.$$$");                                                 *
*****************************************************************************/

int creat_file(char *filename)
 {
  int handle = creatnew(filename,FA_NORMAL);
  close(handle);
  return 0;
 }
